/// <reference types="bn.js" />
import { Connection, PublicKey } from "@solana/web3.js";
import { PDA } from "../pda";
import { CachedContent, CachedValue } from "./fetcher";
import { MarketStateData, KycCertData, OfferDataInfo, TradeData, TradeDataInfo, ConfigData } from "../types";
import { BN } from "@project-serum/anchor";
export declare class EscrowFetcher {
    private readonly fetcher;
    pda: PDA;
    connection: Connection;
    constructor(connection: Connection, pda: PDA, cache?: Record<string, CachedContent<CachedValue>>);
    getConfigData(): Promise<ConfigData | null>;
    getMarketData(tokenMint: PublicKey): Promise<MarketStateData | null>;
    getConfigAddress(): PublicKey;
    getMarketAddress(tokenMint: PublicKey): PublicKey;
    getTradeInfo(tradeId: BN, refresh?: boolean): Promise<TradeDataInfo | null>;
    getTrades(startTradeId: number, endTradeId?: number, refresh?: boolean): Promise<(TradeData | null)[]>;
    getTradesByStatus(tradeStatus: object, side?: object): Promise<TradeDataInfo[]>;
    getTradesByBuyerAddress(address: PublicKey): Promise<TradeDataInfo[]>;
    getTradesBySellerAddress(address: PublicKey): Promise<TradeDataInfo[]>;
    /**
     *
     * @param offerStatus Awaiting || Canceled || OutOfFund
     * @param side Buy || Sell
     * @returns
     */
    getOffersByStatus(offerStatus: object, side?: object): Promise<OfferDataInfo[]>;
    getOffersByMakerAddress(address: PublicKey): Promise<OfferDataInfo[]>;
    getOffer(offerId: BN): Promise<OfferDataInfo | null>;
    getOffers(startOfferId: number, endOfferId?: number, refresh?: boolean): Promise<OfferDataInfo[]>;
    getAllTradesOfAnOffer(offerId: BN): Promise<TradeDataInfo[]>;
    getAllTrades(): Promise<(TradeData | null)[]>;
    getKYCCertData(tokenMint: PublicKey, userPubkey: PublicKey): Promise<KycCertData | null>;
    isArbitrator(userPubkey: PublicKey, tokenMint: PublicKey): Promise<boolean>;
    refreshAll(): Promise<void>;
}
